# -*- coding: utf-8 -*-
from aio_client.base.models import RequestLog
from aio_client.consumer.models import GetConsumerReceipt
from aio_client.consumer.models import GetConsumerResponse
from aio_client.consumer.models import PostConsumerRequest
from aio_client.provider.models import GetProviderReceipt
from aio_client.provider.models import GetProviderRequest
from aio_client.provider.models import PostProviderRequest

from django.contrib import admin


@admin.register(RequestLog)
class RequestLogAdmin(admin.ModelAdmin):
    list_display = ('id', 'timestamp_created', 'request_type')
    list_filter = ('request_type', 'state')
    date_hierarchy = 'timestamp_created'


class MessageLogAdmin(admin.ModelAdmin):
    list_display = ('message_id','origin_message_id', 'display_created',
                    'message_type')
    list_filter = ('message_type','state')
    # date_hierarchy = 'request_id__timestamp_created'
    search_fields = ('message_id', 'origin_message_id')

    def display_created(self, obj):
        return obj.request_id.timestamp_created

    display_created.short_description = u'Создан'


@admin.register(GetProviderRequest)
class GetProviderRequestAdmin(MessageLogAdmin):
    list_filter = ('message_type','state')

# У исходящих запросов В случае ошибки заполняется request_id.state
@admin.register(PostProviderRequest, PostConsumerRequest)
class PostRequestAdmin(MessageLogAdmin):
    list_filter = ('message_type','request_id__error', 'request_id__state')

admin.site.register(
    (
        GetConsumerReceipt,
        GetConsumerResponse,
        GetProviderReceipt,
    ),
    MessageLogAdmin
)