# coding: utf-8
from django.forms.models import model_to_dict

from .exceptions import RequestNotFound
from .exceptions import ResponseNotFound
from .helpers import consumer_post_request
from .models import GetConsumerResponse
from .models import PostConsumerRequest


def push_request(message):
    """Передача сообщения на запрос услуги
    :param message запись aio_client.provider.models.PostProviderRequest
    :return: инстанс класса requests.models.Response
    """
    assert isinstance(message, PostConsumerRequest)
    response = consumer_post_request(message)
    return response


def get_response(message_id):
    """Получение ответа от СМЭВ на Запрос услуги
    :param message_id str идентификаор сообщения, котрое РИС посылает в
    теге message_id при вызове api.push_request
    :return dict со списком полей GetProviderReceipt.LIST_AIO_FIELDS
    """
    qs_response = PostConsumerRequest.objects.filter(
        message_id=message_id)
    if not qs_response.exists():
        raise RequestNotFound(
            message=RequestNotFound.DEFAULT_MSG % message_id)

    receipt = GetConsumerResponse.objects.filter(
        origin_message_id=message_id).order_by('id').last()
    if not receipt:
        raise ResponseNotFound(
            message=ResponseNotFound.DEFAULT_MSG % message_id)
    return model_to_dict(receipt, GetConsumerResponse.LIST_AIO_FIELDS)
