# coding: utf-8
import json

from django.forms.models import model_to_dict

from aio_client.base import RequestTypeEnum
from aio_client.base import _prepare_log
from aio_client.base import send_request

from .models import GetConsumerReceipt
from .models import GetConsumerResponse
from .models import PostConsumerRequest


def consumer_get_requests():
    """Получение всех ответов из очереди СМЭВ
    :return list список словарей с параметрами ответа
    """
    request_log = _prepare_log(RequestTypeEnum.CS_GET)

    response = send_request(request_log)
    request_log.save()
    for message in response.json():
        message['request_id'] = request_log
        GetConsumerResponse.objects.create(**message)
    return response.json()


def consumer_delete_request(message_id):
    """Запрос на удаление полученных ответов
    :param message_id: str идетификатор заявки
    :return инстанс класса requests.models.Response
    """
    request_log = _prepare_log(RequestTypeEnum.CS_DEL)
    request_log.http_body = message_id
    response = send_request(request_log)
    request_log.save()
    return response


def consumer_post_request(request_msg):
    """Передача заявок в СМЭВ
    :param request_msg объект класса PostConsumerRequest
     c параметрами ответа
    :return инстанс класса requests.models.Response
    """
    assert isinstance(request_msg, PostConsumerRequest)
    request_log = _prepare_log(RequestTypeEnum.CS_POST)
    request_log.http_body = model_to_dict(
        request_msg,
        fields=PostConsumerRequest.LIST_AIO_FIELDS)

    response = send_request(request_log)
    request_log.save()
    request_msg.request_id = request_log
    request_msg.save()
    return response


def consumer_get_receipt():
    """Получение ответа СМЭВ по всем отправленным заявкам
    :return dict словарь с параметрами ответа
    """
    request_log = _prepare_log(RequestTypeEnum.CS_GET_R)

    response = send_request(request_log)
    request_log.save()
    for message in response.json():
        message['request_id'] = request_log
        GetConsumerReceipt.objects.create(**message)
    return response.json()


def consumer_delete_receipt(message_id):
    """Запрос на удаление полученных ответов от СМЭВ
    :param message_id: str идентификатор ответа
    :return инстанс класса requests.models.Response
    """
    request_log = _prepare_log(RequestTypeEnum.CS_DEL_R)
    request_log.http_body = message_id
    response = send_request(request_log)
    request_log.save()
    return response
