# coding: utf-8
import json

from django.test import TestCase
import mock

from aio_client.base import RequestTypeEnum
from aio_client.consumer.api import get_response
from aio_client.consumer.api import push_request
from aio_client.consumer.exceptions import RequestNotFound
from aio_client.consumer.exceptions import ResponseNotFound
from aio_client.consumer.helpers import _prepare_log
from aio_client.consumer.models import PostConsumerRequest
from aio_client.consumer.models.tests.factory_consumer import \
    PostConsumerRequestF
import aio_client

from .request_data import TEST_GET_CONSUMER_REQUEST_EXAMPLE
from .request_data import TEST_GET_CONSUMER_REQUEST_OR_MESSAGE_ID
from .request_data import TEST_POST_CONSUMER_REQUEST_MESSAGE_ID


class ApiTestCase(TestCase):

    def setUp(self):
        self.request_type = RequestTypeEnum.PR_GET
        self.request_log = _prepare_log(self.request_type)

    @mock.patch('aio_client.consumer.helpers.send_request')
    def test_push_request(self, sender):
        #отправляем заявку
        request_msg = PostConsumerRequestF(
            message_id=TEST_POST_CONSUMER_REQUEST_MESSAGE_ID)
        sender.return_value = mock.Mock(status_code=200)
        push_request(request_msg)
        self.assertEqual(PostConsumerRequest.objects.all().count(), 1)
        self.assertEqual(
            PostConsumerRequest.objects.all()[0].message_id,
            TEST_POST_CONSUMER_REQUEST_MESSAGE_ID)

    @mock.patch('aio_client.consumer.helpers.send_request')
    def test_get_response_not_found(self, sender):
        """Запрос ответа по заявке, ответ на которую мы не отправляли вызывает
        RequestNotFound
        """
        with self.assertRaises(RequestNotFound):
            get_response(TEST_POST_CONSUMER_REQUEST_MESSAGE_ID)

    @mock.patch('aio_client.consumer.helpers.send_request')
    def test_get_response_response_not_found(self, sender):
        """Запрос ответа по заявке, ответа еще не получили вызывает
        ResponseNotFound"""
        #отправляем заявку
        request_msg = PostConsumerRequestF(
            message_id=TEST_POST_CONSUMER_REQUEST_MESSAGE_ID)
        sender.return_value = mock.Mock(status_code=200)
        push_request(request_msg)

        #запрос ответа по заявке, ответа еще не получили
        with self.assertRaises(ResponseNotFound):
            get_response(TEST_POST_CONSUMER_REQUEST_MESSAGE_ID)

    @mock.patch('aio_client.consumer.helpers.send_request')
    def test_get_response(self, sender):
        """Запрос ответа по заявке"""
        #отправляем заявку
        request_msg = PostConsumerRequestF(
            message_id=TEST_POST_CONSUMER_REQUEST_MESSAGE_ID)
        sender.return_value = mock.Mock(status_code=200)
        push_request(request_msg)

        mock_resp = json.loads(TEST_GET_CONSUMER_REQUEST_EXAMPLE)
        sender.return_value = mock.Mock(ok=True)
        sender.return_value.json.return_value = mock_resp
        # эмуляция работы таска GetAllResponsesConsumerTask
        # запрашиваем ответы
        aio_client.consumer.helpers.consumer_get_requests()
        result = get_response(TEST_POST_CONSUMER_REQUEST_MESSAGE_ID)
        self.assertEqual(result['origin_message_id'],
                         TEST_GET_CONSUMER_REQUEST_OR_MESSAGE_ID)
