# coding: utf-8
import json

from django.test import TestCase
from requests.exceptions import HTTPError
import mock

from aio_client.base import RequestTypeEnum
from aio_client.base.helpers import _prepare_log
from aio_client.consumer.models import GetConsumerReceipt
from aio_client.consumer.models import GetConsumerResponse
from aio_client.consumer.models import PostConsumerRequest
from aio_client.consumer.models.tests.factory_consumer import \
    PostConsumerRequestF
import aio_client

from .request_data import TEST_GET_CONSUMER_RECEIPT_EXAMPLE
from .request_data import TEST_GET_CONSUMER_RECEIPT_MESSAGE_ID
from .request_data import TEST_GET_CONSUMER_REQUEST_EXAMPLE
from .request_data import TEST_GET_CONSUMER_REQUEST_MESSAGE_ID
from .request_data import TEST_GET_CONSUMER_REQUEST_OR_MESSAGE_ID
from .request_data import TEST_POST_CONSUMER_REQUEST_MESSAGE_ID


class HelpersTestCase(TestCase):

    def setUp(self):
        self.request_type = RequestTypeEnum.CS_POST
        self.request_log = _prepare_log(self.request_type)

    @mock.patch('aio_client.consumer.helpers.send_request')
    def test_consumer_post_request(self, sender):
        """Проверка что запрос успешно ушел и попал в БД"""
        request_msg = PostConsumerRequestF(
            message_id=TEST_POST_CONSUMER_REQUEST_MESSAGE_ID)
        sender.return_value = mock.Mock(status_code=200)
        response = aio_client.consumer.helpers.consumer_post_request(
            request_msg)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(PostConsumerRequest.objects.all().count(), 1)
        self.assertEqual(
            PostConsumerRequest.objects.all()[0].message_id,
            TEST_POST_CONSUMER_REQUEST_MESSAGE_ID)

    @mock.patch('aio_client.consumer.helpers.send_request')
    def test_consumer_get_receipt(self, sender):
        """Проверка что запрос успешно ушел и попал в БД"""
        mock_resp = json.loads(TEST_GET_CONSUMER_RECEIPT_EXAMPLE)
        sender.return_value = mock.Mock(ok=True)
        sender.return_value.json.return_value = mock_resp

        response = aio_client.consumer.helpers.consumer_get_receipt()
        # в ответе 1 заявка
        self.assertEqual(len(response), 1)
        # в таблице 1 заявка
        self.assertEqual(GetConsumerReceipt.objects.all().count(), 1)
        self.assertEqual(response[0]['message_id'],
                         TEST_GET_CONSUMER_RECEIPT_MESSAGE_ID)

    @mock.patch('aio_client.consumer.helpers.send_request')
    def test_consumer_delete_receipt(self, sender):
        """Проверка что запрос успешно ушел и попал в БД"""
        sender.return_value = mock.Mock(status_code=200)
        response = aio_client.consumer.helpers.consumer_delete_receipt(
            TEST_GET_CONSUMER_RECEIPT_MESSAGE_ID)
        self.assertEqual(response.status_code, 200)

    @mock.patch('aio_client.consumer.helpers.send_request')
    def test_consumer_get_requests(self, sender):
        """Если ответ не пуст, то сообщение должно попасть в БД"""
        mock_resp = json.loads(TEST_GET_CONSUMER_REQUEST_EXAMPLE)
        sender.return_value = mock.Mock(ok=True)
        sender.return_value.json.return_value = mock_resp

        response = aio_client.consumer.helpers.consumer_get_requests()
        # в ответе 1 заявка
        self.assertEqual(len(response), 1)
        # в таблице 1 заявка
        self.assertEqual(GetConsumerResponse.objects.all().count(), 1)

        self.assertEqual(response[0]['origin_message_id'],
                         TEST_GET_CONSUMER_REQUEST_OR_MESSAGE_ID)

    @mock.patch('aio_client.consumer.helpers.send_request')
    def test_consumer_delete_request_not_exists(self, sender):
        mock_resp = mock.Mock(
            status_code=404, raise_for_status=HTTPError("Not Found"))
        sender.return_value = mock_resp
        response = aio_client.consumer.helpers.consumer_delete_request('0')
        self.assertEqual(response.status_code, 404)

    @mock.patch('aio_client.consumer.helpers.send_request')
    def test_consumer_delete_request(self, sender):
        sender.return_value = mock.Mock(status_code=200)
        response = aio_client.consumer.helpers.consumer_delete_request(
            TEST_GET_CONSUMER_REQUEST_MESSAGE_ID)
        self.assertEqual(response.status_code, 200)
