# -*- coding: utf-8 -*-
from django.db import models

from aio_client.base.models import GetReceipt
from aio_client.base.models import RequestMessage


class GetProviderRequest(RequestMessage):
    """ответ на запрос получение всех заявок к РИС"""
    # Список полей приходящих из АИО
    LIST_AIO_FIELDS = (
        "message_id",
        "origin_message_id",
        "body",
        "attachments",
        "is_test_message",
        "replay_to",
        "message_type")
    is_test_message = models.BooleanField(
        default=False, verbose_name=u'Признак тестового взаимодействия')
    replay_to = models.CharField(
        max_length=100, verbose_name=u'Индекс сообщения в СМЭВ')


class PostProviderRequest(RequestMessage):
    """ответ РИС как постащика на заявку из АИО"""
    # Список полей в сообщение, которые ожидает АИО
    LIST_AIO_FIELDS = (
        "origin_message_id",
        "body",
        "message_type",
        "attachments",
        "content_failure_code",
        "content_failure_comment",
        "replay_to",
        "is_test_message"
    )

    ACCESS_DENIED = 'ACCESS_DENIED'
    UNKNOWN_REQUEST_DESCRIPTION = 'UNKNOWN_REQUEST_DESCRIPTION'
    NO_DATA = 'NO_DATA'
    FAILURE = 'FAILURE'

    FAILURE_CODES = (
        (ACCESS_DENIED, 'access denied'),
        (UNKNOWN_REQUEST_DESCRIPTION, 'unknown request description'),
        (NO_DATA, 'no data'),
        (FAILURE, 'failure'),
    )
    # используется как "обратный адрес" при передаче ответа на заявку
    replay_to = models.CharField(
        max_length=4000, verbose_name=u'Индекс сообщения в СМЭВ')
    content_failure_code = models.CharField(
        max_length=50,
        choices=FAILURE_CODES,
        null=True,
        blank=True,
        verbose_name=u'Код причины отказа'
    )
    content_failure_comment = models.TextField(
        default='', blank=True, verbose_name=u'Пояснение причины отказа')
    is_test_message = models.BooleanField(
        default=False, verbose_name=u'Признак тестового взаимодействия')


class GetProviderReceipt(GetReceipt):
    """Ответы из очереди СМЭВ"""
    # Список полей приходящих из АИО
    LIST_AIO_FIELDS = (
        "message_id",
        "error",
        "origin_message_id",
        "fault",
        "message_type",
    )
